@@ -51,7 +51,6 @@ import org.apache.ws.security.WSSConfig;
 import org.apache.ws.security.WSSecurityEngine;
 import org.apache.ws.security.WSSecurityEngineResult;
 import org.apache.ws.security.WSSecurityException;
-import org.apache.ws.security.components.crypto.Crypto;
 import org.apache.ws.security.handler.RequestData;
 import org.apache.ws.security.handler.WSHandlerConstants;
 import org.apache.ws.security.handler.WSHandlerResult;
@@ -69,6 +68,7 @@ public class WSS4JInInterceptor extends AbstractWSS4JInterceptor {
     public static final String TIMESTAMP_RESULT = "wss4j.timestamp.result";
     public static final String SIGNATURE_RESULT = "wss4j.signature.result";
     public static final String PRINCIPAL_RESULT = "wss4j.principal.result";
+    public static final String BST_RESULT = "wss4j.bst.result";
     public static final String PROCESSOR_MAP = "wss4j.processor.map";
 
     private static final Logger LOG = LogUtils.getL7dLogger(WSS4JInInterceptor.class);
@@ -197,18 +197,9 @@ public class WSS4JInInterceptor extends AbstractWSS4JInterceptor {
             // Extract the signature action result from the action vector
             WSSecurityEngineResult actionResult = WSSecurityUtil
                 .fetchActionResult(wsResult, WSConstants.SIGN);
-
-            if (actionResult != null) {
-                X509Certificate returnCert = (X509Certificate)actionResult
-                    .get(WSSecurityEngineResult.TAG_X509_CERTIFICATE);
-
-                if (returnCert != null && !verifyTrust(returnCert, reqData)) {
-                    LOG.warning("The certificate used for the signature is not trusted");
-                    throw new WSSecurityException(WSSecurityException.FAILED_CHECK);
-                }
-                msg.put(SIGNATURE_RESULT, actionResult);
-            }
-
+            verifyTrust(actionResult, reqData);
+            msg.put(SIGNATURE_RESULT, actionResult);            
+            
             /*
              * Perform further checks on the timestamp that was transmitted in
              * the header. In the following implementation the timestamp is
@@ -338,22 +329,6 @@ public class WSS4JInInterceptor extends AbstractWSS4JInterceptor {
         }
         return cbHandler;
     }
-    public Crypto loadSignatureCrypto(RequestData reqData) 
-        throws WSSecurityException {
-        try {
-            return super.loadSignatureCrypto(reqData);
-        } catch (Exception ex) {
-            return null;
-        }
-    }
-    protected Crypto loadDecryptionCrypto(RequestData reqData) 
-        throws WSSecurityException {
-        try {
-            return super.loadDecryptionCrypto(reqData);
-        } catch (Exception ex) {
-            return null;
-        }
-    }
 
 
     
@@ -436,5 +411,28 @@ public class WSS4JInInterceptor extends AbstractWSS4JInterceptor {
         }
         return fault;
     }
-    
+
+    private void verifyTrust(WSSecurityEngineResult actionResult, RequestData reqData)
+        throws WSSecurityException {
+        if (actionResult != null) {
+            X509Certificate[] certs = 
+                (X509Certificate[])actionResult.get(WSSecurityEngineResult.TAG_X509_CERTIFICATES);
+            if (certs != null && certs.length > 0) {
+                if (!verifyTrust(certs, reqData)) {
+                    LOG.warning("The certificate chain used for the signature is not trusted");
+                    throw new WSSecurityException(WSSecurityException.FAILED_CHECK);
+                }
+            } else {                
+                X509Certificate returnCert = (X509Certificate)actionResult
+                    .get(WSSecurityEngineResult.TAG_X509_CERTIFICATE);
+                
+                if (returnCert != null && !verifyTrust(returnCert, reqData)) {
+                    LOG.warning("The certificate used for the signature is not trusted");
+                    throw new WSSecurityException(WSSecurityException.FAILED_CHECK);
+                }
+            }
+        }        
+    }
+
 }
+
